// Copyright (C) 2024  Nexedi SA and Contributors.
//                     Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

//go:build !WEAK_BUGGY_CRASHES_GC

// Package weak is stub for weak references.
//
// It provides weak references API, but the references are full strong, not weak, pointers.
//
// It should be removed once std package weak is, hopefully, provided in Go standard library.
// See https://github.com/golang/go/issues/67552 for details.
package weak

const Stub = true

type Ref[T any] struct {
	ptr *T
}

func NewRef[T any](obj *T) *Ref[T] {
	return &Ref[T]{obj}
}

func (w *Ref[T]) Get() (obj *T) {
	return w.ptr
}
