// Copyright (C) 2024  Nexedi SA and Contributors.
//                     Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

package zodb_test

import (
	"testing"

	"lab.nexedi.com/kirr/neo/go/zodb"
	pickle "github.com/kisielk/og-rek"

	"lab.nexedi.com/kirr/neo/go/internal/xtesting"
	assert "github.com/stretchr/testify/require"
)


// ztestdataReg maintains registry of all entries under testdata/ .
var  ztestdataReg = xtesting.ZTestDataRegistry[struct{}]{}
type ZTestData    = xtesting.ZTestData[struct{}]


// Verify that Map and List can load from data saved by py.
// TODO go saves Map/List -> py loads and checks.
func TestPersistentMapListLoad(t *testing.T) {
	ztestdataReg.RunWithEach(t, _TestPersistentMapListLoad)
}
func _TestPersistentMapListLoad(t0 *testing.T, z *ZTestData) {
	assert := assert.New(t0)

	tdb := zodb.OpenTestDB(t0, z.Path("data.fs"))
	defer tdb.Close()

	t := tdb.Open(&zodb.ConnOptions{})
	X := xtesting.FatalIf(t0)

	xroot := t.GetAny(0)
	root, ok := xroot.(*zodb.Map)
	if !ok {
		t.Fatalf("root: got %T  ; expect Map", xroot)
	}

	t.PActivate(root)
	defer root.PDeactivate()

	// see py/pydata-gen-testdata
	assert.Equal(root.Len(), 3)
	assert.Equal(root.Get("int1"), int64(1))

	xabc := root.Get("strABC")
	abc, err := pickle.AsString(xabc); X(err)
	assert.Equal(abc, "abc")

	xplist := root.Get("plist")
	plist, ok := xplist.(*zodb.List)
	if !ok {
		t.Fatalf("plist: got %T  ; expect List", xplist)
	}

	t.PActivate(plist)
	defer plist.PDeactivate()

	assert.Equal(len(plist.Data), 3)
	xa := plist.Data[0]
	a, err := pickle.AsString(xa); X(err)
	assert.Equal(a, "a")

	assert.Equal(plist.Data[1], int64(1))
	assert.Equal(plist.Data[2], pickle.None{})
}
